#ifndef PPSSUMM_INSTRUMENT
#define PPSSUMM_INSTRUMENT

#include <string>
#include "string_algo.h"

class Instrument
{
    public:
    Instrument() : _instrument("") { }
    Instrument( const char* s ) : _instrument( s ) { _init(); }
    Instrument( const std::string& s ) : _instrument( s ) 
    {
    _init(); 

    }
    Instrument( const Instrument& s ) : _instrument( s.asString() ) 
    {
    _init(); 
    }

    Instrument operator=(const Instrument& s)
    {
        _instrument = s.asString();
        return *this;
    }
    Instrument operator=(const char* s)
    {
        _instrument = s;
	_init();
        return *this;
    }
    Instrument operator=(const std::string& s)
    {
        _instrument = s;
	_init();
        return *this;
    }

    bool operator==(const Instrument& s) const
    {
	return _instrument == s._instrument;
    }
    bool operator<(const Instrument& s) const
    {    
	return _instrument < s._instrument;
    }

    std::string asString() const { return _instrument; }
    std::string longname() const { return _instrument; }
    std::string shortname() const
    {
	std::string r("");
	if (_instrument == "EPIC") r = "EP";
	else if (_instrument == "EMOS") r = "EM";
	else if (_instrument == "EMOS1") r = "M1";
	else if (_instrument == "EMOS2") r = "M2";
	else if (_instrument == "EPN") r = "PN";
	else if (_instrument == "RGS") r = "RG";
	else if (_instrument == "RGS1") r = "R1";
	else if (_instrument == "RGS2") r = "R2";
	else if (_instrument == "OBSERVATION") r = "OB";
	else if (_instrument == "CATALOGUE") r = "CA";
	else r = _instrument;
	return r;
    }

    /* Switch between writing short or long names */
    void shortnames()
    { _shortname = 1; }

    void longnames()
    { _shortname = 0; }

    friend std::ostream &operator<<(std::ostream& os , const Instrument& s)
    {
	if (_shortname)
	{
	    os << s.shortname();
	}
	else
	{
	    os << s.asString();
	}
	return os;
    }

    std::string page() const
    {
	std::string r("");
	if (_instrument == "EPIC") r = "epxpage";
	else if (_instrument == "EMOS") r = "epxpage";
	else if (_instrument == "EMOS1") r = "epxpage";
	else if (_instrument == "EMOS2") r = "epxpage";
	else if (_instrument == "EPN") r = "epxpage";
	else if (_instrument == "OM") r = "omxpage";
	else if (_instrument == "RGS") r = "rgxpage";
	else if (_instrument == "RGS1") r = "rgxpage";
	else if (_instrument == "RGS2") r = "rgxpage";
	else if (_instrument == "OBSERVATION") r = "ppspage";
	else if (_instrument == "CATALOGUE") r = "catpage";
	else r = "";
	return r;
    }

    std::string pagename() const
    {
	std::string r = page();
	return ( r.size()
	    ? ppssumm_internal::stripdir( stringParameter( r ) )
	    : r
	);
    }

    int asInteger() const
    {
	// Numerical ID used by operator<
	int r;
	if (_instrument == "EPIC") r = 2;
	else if (_instrument == "EMOS") r = 3;
	else if (_instrument == "EMOS1") r = 4;
	else if (_instrument == "EMOS2") r = 5;
	else if (_instrument == "EPN") r = 6;
	else if (_instrument == "OM") r = 7;
	else if (_instrument == "RGS") r = 8;
	else if (_instrument == "RGS1") r = 9;
	else if (_instrument == "RGS2") r = 10;
	else if (_instrument == "OBSERVATION") r = 1;
	else if (_instrument == "CATALOGUE") r = 11;
	else r = 12;
	return r;
    }

    private:
    static bool _shortname;
    std::string _instrument;
    void _init()
    {
	if (_instrument == "EP") _instrument = "EPIC";
	else if (_instrument == "EM") _instrument = "EMOS";
	else if (_instrument == "M1") _instrument = "EMOS1";
	else if (_instrument == "M2") _instrument = "EMOS2";
	else if (_instrument == "PN") _instrument = "EPN";
	else if (_instrument == "RG") _instrument = "RGS";
	else if (_instrument == "R1") _instrument = "RGS1";
	else if (_instrument == "R2") _instrument = "RGS2";
	else if (_instrument == "OB") _instrument = "OBSERVATION";
	else if (_instrument == "CA") _instrument = "CATALOGUE";
    }
};


//bool Instrument::_shortname = 0;

class MakeInstrument
{
    public:
    Instrument operator()(const Instrument& s) { return Instrument( s ); };
    Instrument operator()(  const char* s ) { return Instrument(s); };
    Instrument operator()( const std::string& s ) { return Instrument(s); };
};
#endif
