#ifndef PPSSUMM_OSW_ID
#define PPSSUMM_OSW_ID

#include <string>
#include <sstream>
#include <cctype>
#include <cstdlib>
#include <functional>
#include "string_algo.h"

class Osw_id
{
    public:
    Osw_id() : _number(-1) , _filter("") {}
    Osw_id( const long& s ) : _number(s) , _filter("") {}
    explicit Osw_id( const char* s ) : _number(-1) , _filter("") { set(s); }
    Osw_id( const std::string& s ) : _number(-1) , _filter("") { set(s);}

    void get ( long& i ) const { i = _number; }
    void get ( std::string& s ) const { s = _filter; }

    Osw_id set( const long& s ) 
    {
	Osw_id tmp = *this;
	_filter = "";
	_number = s;
	return tmp; 
    } 

    Osw_id set( const std::string& s ) 
    {
	Osw_id tmp = *this;

	ppssumm_internal::notBlank notblank;

	if ( notblank(s) )
	{
	    std::string::size_type numpos = s.find_first_not_of("0123456789");
	    if (numpos != std::string::npos )
	    {
		_filter = s;
		_number = -1;
	    }
	    else
	    {
		_number = strtol( s.c_str() , '\0' , 10 );
		_filter = "";
	    }
	}
	else
	{
	    _filter == "";
	    _number = -1;
	}
	return tmp; 
    } 

    // Operators
    bool operator==(const Osw_id& s) const
    {
	bool r = false;
	if ( _number > -1 && s._number > -1 )
	{
	    r = _number < s._number;
	}
	else if ( _filter.length() && s._filter.length() )
	{
	    r = _filter < s._filter;
	}
	else if ( _filter.length() && s._number > -1 )
	{
	    r = false;
	}
	else if ( _number > -1 && s._filter.length() )
	{
	    r = true;
	}
	return r;
    }
    bool operator<(const Osw_id& s) const
    { 
	bool r = false;
	if ( _number == -1 && !_filter.length() )
	{
	    r = true;
	}
	else if ( _number > -1 && s._number > -1 )
	{
	    r = _number < s._number;
	}
	else if ( _filter.length() && s._filter.length() )
	{
	    r = _filter < s._filter;
	}
	else if ( _filter.length() && s._number > -1 )
	{
	    r = false;
	}
	else if ( _number > -1 && s._filter.length() )
	{
	    r = true;
	}
	return r;
    }

    std::string osw_id() const
    {
	stringstream ss;
	if (_number > -1)
	{
	    ss << _number;
	}
	else 
	{
	    ss << "&nbsp;";
	}
	return ss.str();
    }
    std::string filter() const
    {
	stringstream ss;
	if (_filter.size() )
	{
	    ss << _filter;
	}
	else 
	{
	    ss << "&nbsp;";
	}
	return ss.str();
    }

    friend std::ostream &operator<<(std::ostream& os , const Osw_id& s) 
    {
	if (s._number > -1)
	{
	    os << s._number;
	}
	else if (s._filter.size() )
	{
	    os << s._filter;
	}
	else 
	{
	    os << "&nbsp;";
	}

	return os;
    }

    private:
    long _number;
    std::string _filter;
};

#endif

