#ifndef PPSSUMM_PRODUCT_SUMMARY
#define PPSSUMM_PRODUCT_SUMMARY

#include <vector>
#include <string>
#include <iostream>
#include <algorithm>
#include <iterator>
#include "ppssumm_typedef.h"
#include "PpssummCommon.h"
#include "Pindex.h"
#include "Eindex.h"
#include "Omindex.h"
#include "Rgsindex.h"
#include "Ppsindex.h"
#include "ppssumm_algo.h"
#include "string_algo.h"
#include "key_algo.h"
#include "STime.h"
#include "Dal.h"
#include "param.h"
#include "Tracer.h"
#include "errstr.h"

class ProductSummary
{
    public:
    ProductSummary( const Pindex& pindex
	,const Epicexposuresprocessed* epic
	,const Rgsexposuresprocessed* rgs
	,const Omexposuresprocessed* om
    ) : _genericproduct(pindex)
    {
	_epicexposuresprocessed = epic;
	_rgsexposuresprocessed = rgs;
	_omexposuresprocessed = om;
    }
    
    friend ostream& operator<<( ostream& os , const ProductSummary& p )
    {
	// Page menu
	os  << "<div id=\"pagemenu\">\n"
	    << "<ul>\n"
	    << "<li><a href=\"#epicexposuresprocessed\">EPIC Exposures Processed</a></li>\n"
	    << "<li><a href=\"#omexposuresprocessed\">OM Exposures Processed</a></li>\n"
	    << "<li><a href=\"#rgsexposuresprocessed\">RGS Exposures Processed</a></li>\n"
	    << "<li><a href=\"#genericproduct\">Generic Products</a></li>\n"
	    << "</ul>\n"
	    << "<br clear=\"left\" />"
	    << "<hr />\n"
	    << "</div>"
	;

	// EPIC exposures
	os << "<div id=\"widetable\">" << "\n";
	os << "<a name=\"epicexposuresprocessed\">&nbsp;</a>" << "\n";
	os << "<h3>EPIC Exposures processed by PPS</h3>" << "\n";
	if (p._epicexposuresprocessed->size())
	{
	    // Exposures processed table
	    os << "<table class=\"epicexposureprocessed_table\">" << "\n";
	    os << *(p._epicexposuresprocessed);
	    os << "</table>" << "\n";
	}
	else
	{
	    os << "<p class=\"noprod\">No EPIC exposures processed by the PPS.</p>" << "\n";

	}
	os << "<hr />" << "\n";
	os << "</div>" << "\n";

	// OM exposures
	os << "<div id=\"widetable\">" << "\n";
	os << "<a name=\"omexposuresprocessed\">&nbsp;</a>" << "\n";
	os << "<h3>OM Exposures processed by PPS</h3>" << "\n";
	if (p._omexposuresprocessed->size())
	{
	    // Exposures processed table
	    os << "<table class=\"omexposureprocessed_table\">" << "\n";
	    os << *(p._omexposuresprocessed);
	    os << "</table>" << "\n";
	}
	else
	{
	    os << "<p class=\"noprod\">No OM exposures processed by the PPS.</p>" << "\n";

	}
	os << "<hr />" << "\n";
	os << "</div>" << "\n";

	// RGS exposures
	os << "<div id=\"widetable\">" << "\n";
	os << "<a name=\"rgsexposuresprocessed\">&nbsp;</a>" << "\n";
	os << "<h3>RGS Exposures processed by PPS</h3>" << "\n";
	if (p._rgsexposuresprocessed->size())
	{
	    // Exposures processed table
	    os << "<table class=\"rgsexposureprocessed_table\">" << "\n";
	    os << *(p._rgsexposuresprocessed);
	    os << "</table>" << "\n";
	}
	else
	{
	    os << "<p class=\"noprod\">No RGS exposures processed by the PPS.</p>" << "\n";
	}
	os << "<hr />" << "\n";
	os << "</div>" << "\n";

	// Generic Products
	os  << "<div id=\"normaltable\">"  << "\n"
	    << "<a name=\"genericproduct\">&nbsp;</a>"
	    << "<h3>Generic Products</h3>"
	;
	if (p._genericproduct.size())
	{
	    os	<< "<table class=\"genericproduct_table\">"
		<< p._genericproduct
		<< "</table>"
	    ;
	}
	else
	{
	    os << "<p class=\"noprod\">No generic products were produced.</p>";
	}
	os << "<hr />";
	os << "</div>";
	return os;
    }

    private:
	Genericproduct _genericproduct;
	const Epicexposuresprocessed* _epicexposuresprocessed;
	const Rgsexposuresprocessed* _rgsexposuresprocessed;
	const Omexposuresprocessed* _omexposuresprocessed;
};
#endif
