#ifndef PPSSUMMSTRING_ALGO
#define PPSSUMMSTRING_ALGO
#include "ppssumm_typedef.h"
#include <string>
#include <cmath>
#include <cerrno>
#include <cctype>
#include <sstream>
#include <functional>
#include <iterator>
#include "Dal.h"
#include "param.h"
#include "errstr.h"

namespace ppssumm_internal
{
    std::string stripdir( const std::string& );
    std::string truncTime( const std::string& ); 
  
    char toUpper( const char );
    char toLower( const char );

    std::string pagename(const std::string& );
    /* 
     * WARNING
     * 
     * <string> with g++ 2.95.3 #define 's CP to be 32
     * Don't use it ! Especially not as a template parameter.
     * Aaaaaaaaaaaaaaaaaaaaargh!
     * 
     */


    template< typename TP >
    class toString : unary_function< TP , std::string > 
    {
        public:
        toString(const int p=0) : _precision(p) {}
        std::string operator()( const TP& v ) const 
        {
            std::stringstream s;
            if ( _precision ) s.precision(_precision);
            s << v;
            return s.str();
        }
        
        private:
        int _precision;
    };

    /*
     * I know there is a toString in stingutil.h but it doesn't work
     * for bool8 or DString.  It returns ^A (true) and ^@ (false) 
     * and leaves trailing ^@ on the end of strings read from FIT files.
     */

    template<>
    class toString< std::string > : unary_function< std::string , std::string > 
    {
        // Cut a string at first '\0'
        // This is necesary for strings pulled from FIT files
        // as they are with '\0'
        public:
        std::string operator()( const std::string& v ) const 
        {
            return v.substr( 0 , v.find_first_of( '\0' ));
        }
    };

    template<>
    class toString< bool8 > : unary_function< bool8 , std::string > 
    {
        // Turn a boolean into text.  Usuallly Y|N, 1|0 , True|False
        public:
        std::string operator()( const bool8 v ) const 
        {
            return v ? "Y" : "N" ;
        }
    };


    class notBlank : unary_function< std::string  , bool  > 
    {
        /* 
         * Simple function for use with stl algorithms.
         * Return true if the input string contains elements 
         * which are not whitespace.
         */
        public:
        bool operator()( const std::string& s ) const 
        {
        bool rtn;
        if ( s.empty() ) rtn = false;
        else rtn =  (s.find_first_not_of(" \t\n\b\f\r\v") != std::string::npos) ;
        return rtn;
        }
    };

}
#endif
